@@ -93,8 +93,6 @@ static void sdl_add_audio_sample (int sample_pos, float left, float right)
 
 static int audio_started = 0;
 
-void gegl_create_chain (char **ops, GeglNode *iter, GeglNode *proxy);
-
 /*  this structure contains the full application state, and is what
  *  re-renderings of the UI is directly based on.
  */
@@ -102,12 +100,9 @@ typedef struct _State State;
 struct _State {
   void      (*ui) (Mrg *mrg, void *state);
   Mrg        *mrg;
-
   char       *path;
   char       *save_path;
-
   GList      *paths;
-
   GeglBuffer *buffer;
   GeglNode   *gegl;
   GeglNode   *sink;
@@ -115,36 +110,27 @@ struct _State {
   GeglNode   *load;
   GeglNode   *save;
   GeglNode   *active;
-
   GeglNode   *rotate;
   int         rev;
   float       u, v;
   float       scale;
   int         show_actions;
   int         show_controls;
-
   float       render_quality;
   float       preview_quality;
-
   int         controls_timeout;
-
   char      **ops; // the operations part of the commandline, if any
-
   float       slide_pause;
   int         slide_enabled;
   int         slide_timeout;
-
   GeglNode   *gegl_decode;
   GeglNode   *decode_load;
   GeglNode   *decode_store;
-
   int         is_video;
   int         frame_no;
-
   int         prev_frame_played;
 };
 
-
 typedef struct ActionData {
   const char *label;
   float priority; /* XXX: not yey used, should force scale/crop to happen early */
@@ -154,13 +140,13 @@ typedef struct ActionData {
 /* white-list of operations useful for improving/altering photos
  */
 ActionData actions[]={
-  {"rotate",            10,  "gegl:rotate"},
-  {"crop",              20,  "gegl:crop"},
-  {"color temperature", 50,  "gegl:color-temperature"},
-  {"exposure",          60,  "gegl:exposure"},
-  //{"levels",            60,  "gegl:levels"},
-  //{"threshold",         70,  "gegl:threshold"},
-  {"unsharp-mask",         70,  "gegl:unsharp-mask"},
+  {"rotate",            10, "gegl:rotate"},
+  {"crop",              20, "gegl:crop"},
+  {"color temperature", 50, "gegl:color-temperature"},
+  {"exposure",          60, "gegl:exposure"},
+  //{"levels",          60, "gegl:levels"},
+  //{"threshold",       70, "gegl:threshold"},
+  {"unsharp-mask",      70, "gegl:unsharp-mask"},
   {NULL, 0, NULL}, /* sentinel */
 };
 
@@ -178,11 +164,11 @@ static int is_gegl_path (const char *path);
 static void contrasty_stroke (cairo_t *cr);
 
 static void mrg_gegl_blit (Mrg *mrg,
-			   float x0, float y0,
-			   float width, float height,
-			   GeglNode *node,
-			   float u, float v,
-			   float scale,
+                           float x0, float y0,
+                           float width, float height,
+                           GeglNode *node,
+                           float u, float v,
+                           float scale,
                            float preview_multiplier);
 
 gchar *get_thumb_path (const char *path);
@@ -210,81 +196,48 @@ static void load_path (State *o);
 
 static void go_next (State *o);
 static void go_prev (State *o);
-static void go_parent (State *o);
-
-static void get_coords (State *o, float screen_x, float screen_y, float *gegl_x, float *gegl_y);
-static void leave_editor (State *o);
-static void drag_preview (MrgEvent *e);
-static void load_into_buffer (State *o, const char *path);
-static GeglNode *locate_node (State *o, const char *op_name);
-static void zoom_to_fit (State *o);
-static void center (State *o);
-static void zoom_to_fit_buffer (State *o);
-
-static void go_next_cb   (MrgEvent *event, void *data1, void *data2);
-static void go_prev_cb   (MrgEvent *event, void *data1, void *data2);
-static void go_parent_cb (MrgEvent *event, void *data1, void *data2);
-static void zoom_fit_cb  (MrgEvent *e, void *data1, void *data2);
-static void pan_left_cb  (MrgEvent *event, void *data1, void *data2);
-static void pan_right_cb (MrgEvent *event, void *data1, void *data2);
-static void pan_down_cb  (MrgEvent *event, void *data1, void *data2);
-static void pan_up_cb    (MrgEvent *event, void *data1, void *data2);
-static void preview_more_cb (MrgEvent *event, void *data1, void *data2);
-static void preview_less_cb (MrgEvent *event, void *data1, void *data2);
-static void zoom_1_cb               (MrgEvent *event, void *data1, void *data2);
-static void zoom_in_cb              (MrgEvent *event, void *data1, void *data2);
-static void zoom_out_cb             (MrgEvent *event, void *data1, void *data2);
-static void toggle_actions_cb       (MrgEvent *event, void *data1, void *data2);
-static void toggle_fullscreen_cb    (MrgEvent *event, void *data1, void *data2);
-static void activate_op_cb          (MrgEvent *event, void *data1, void *data2);
-static void disable_filter_cb       (MrgEvent *event, void *data1, void *data2);
-static void apply_filter_cb         (MrgEvent *event, void *data1, void *data2);
-static void discard_cb              (MrgEvent *event, void *data1, void *data2);
-static void save_cb                 (MrgEvent *event, void *data1, void *data2);
-static void toggle_show_controls_cb (MrgEvent *event, void *data1, void *data2);
-
-static void gegl_ui       (Mrg *mrg, void *data);
-int         mrg_ui_main   (int argc, char **argv, char **ops);
-void        gegl_meta_set (const char *path, const char *meta_data);
-char *      gegl_meta_get (const char *path); 
-
-
-static void on_viewer_motion (MrgEvent *e, void *data1, void *data2);
-
-static int str_has_image_suffix (char *path)
-{
-  return g_str_has_suffix (path, ".jpg") ||
-         g_str_has_suffix (path, ".png") ||
-         g_str_has_suffix (path, ".JPG") ||
-         g_str_has_suffix (path, ".PNG") ||
-         g_str_has_suffix (path, ".jpeg") ||
-         g_str_has_suffix (path, ".JPEG") ||
-         g_str_has_suffix (path, ".CR2") ||
-         g_str_has_suffix (path, ".exr");
-}
-
-static int str_has_video_suffix (char *path)
-{
-  return g_str_has_suffix (path, ".avi") ||
-         g_str_has_suffix (path, ".AVI") ||
-         g_str_has_suffix (path, ".mp4") ||
-         g_str_has_suffix (path, ".mp3") ||
-         g_str_has_suffix (path, ".mpg") ||
-         g_str_has_suffix (path, ".ogv") ||
-         g_str_has_suffix (path, ".MPG") ||
-         g_str_has_suffix (path, ".webm") ||
-         g_str_has_suffix (path, ".MP4") ||
-         g_str_has_suffix (path, ".mkv") ||
-         g_str_has_suffix (path, ".gif") ||
-         g_str_has_suffix (path, ".GIF") ||
-         g_str_has_suffix (path, ".MKV") ||
-         g_str_has_suffix (path, ".mov") ||
-         g_str_has_suffix (path, ".ogg");
-}
+static void go_parent                 (State *o);
+static void get_coords                (State *o, float  screen_x, float screen_y,
+                                                 float *gegl_x,   float *gegl_y);
+static void leave_editor              (State *o);
+static void drag_preview              (MrgEvent *e);
+static void load_into_buffer          (State *o, const char *path);
+static GeglNode *locate_node          (State *o, const char *op_name);
+static void zoom_to_fit               (State *o);
+static void center                    (State *o);
+static void zoom_to_fit_buffer        (State *o);
+static void go_next_cb                (MrgEvent *event, void *data1, void *data2);
+static void go_prev_cb                (MrgEvent *event, void *data1, void *data2);
+static void go_parent_cb              (MrgEvent *event, void *data1, void *data2);
+static void zoom_fit_cb               (MrgEvent *e, void *data1, void *data2);
+static void pan_left_cb               (MrgEvent *event, void *data1, void *data2);
+static void pan_right_cb              (MrgEvent *event, void *data1, void *data2);
+static void pan_down_cb               (MrgEvent *event, void *data1, void *data2);
+static void pan_up_cb                 (MrgEvent *event, void *data1, void *data2);
+static void preview_more_cb           (MrgEvent *event, void *data1, void *data2);
+static void preview_less_cb           (MrgEvent *event, void *data1, void *data2);
+static void zoom_1_cb                 (MrgEvent *event, void *data1, void *data2);
+static void zoom_in_cb                (MrgEvent *event, void *data1, void *data2);
+static void zoom_out_cb               (MrgEvent *event, void *data1, void *data2);
+static void toggle_actions_cb         (MrgEvent *event, void *data1, void *data2);
+static void toggle_fullscreen_cb      (MrgEvent *event, void *data1, void *data2);
+static void activate_op_cb            (MrgEvent *event, void *data1, void *data2);
+static void disable_filter_cb         (MrgEvent *event, void *data1, void *data2);
+static void apply_filter_cb           (MrgEvent *event, void *data1, void *data2);
+static void discard_cb                (MrgEvent *event, void *data1, void *data2);
+static void save_cb                   (MrgEvent *event, void *data1, void *data2);
+static void toggle_show_controls_cb   (MrgEvent *event, void *data1, void *data2);
+static void gegl_ui                   (Mrg *mrg, void *data);
+int         mrg_ui_main               (int argc, char **argv, char **ops);
+void        gegl_meta_set             (const char *path, const char *meta_data);
+char *      gegl_meta_get             (const char *path); 
+static void on_viewer_motion          (MrgEvent *e, void *data1, void *data2);
+int         gegl_str_has_image_suffix (char *path);
+int         gegl_str_has_video_suffix (char *path);
 
 static int str_has_visual_suffix (char *path)
 {
-  return str_has_image_suffix (path) || str_has_video_suffix (path);
+  return gegl_str_has_image_suffix (path) || gegl_str_has_video_suffix (path);
 }
 
 static void populate_path_list (State *o)
@@ -346,7 +299,6 @@ static State *hack_state = NULL;  // XXX: this shoudl be factored away
 
 char **ops = NULL;
 
-
 static void open_audio (int frequency)
 {
   SDL_AudioSpec spec = {0};
@@ -408,11 +360,14 @@ int mrg_ui_main (int argc, char **argv, char **ops)
       return -1;
     }
 
-
   load_path (&o);
   mrg_set_ui (mrg, gegl_ui, &o);
   hack_state = &o;  
   on_viewer_motion (NULL, &o, NULL);
+
+  if (o.ops)
+    o.active = gegl_node_get_producer (o.sink, "input", NULL);
+
   mrg_main (mrg);
 
   g_object_unref (o.gegl);
@@ -463,6 +418,7 @@ static void on_pan_drag (MrgEvent *e, void *data1, void *data2)
     o->v -= (e->delta_y );
     mrg_queue_draw (e->mrg, NULL);
   }
+  mrg_event_stop_propagate (e);
   drag_preview (e);
 }
 
@@ -479,6 +435,52 @@ static void prop_double_drag_cb (MrgEvent *e, void *data1, void *data2)
   gegl_node_set (node, pspec->name, value, NULL);
    
   drag_preview (e);
+  mrg_event_stop_propagate (e);
+
+  mrg_queue_draw (e->mrg, NULL);
+}
+
+static void prop_int_drag_cb (MrgEvent *e, void *data1, void *data2)
+{
+  GeglNode *node = data1;
+  GParamSpec *pspec = data2;
+  GeglParamSpecInt *gspec = data2;
+  gint value = 0.0;
+  float range = gspec->ui_maximum - gspec->ui_minimum;
+
+  value = e->x / mrg_width (e->mrg) * range + gspec->ui_minimum;
+  gegl_node_set (node, pspec->name, value, NULL);
+   
+  drag_preview (e);
+  mrg_event_stop_propagate (e);
+
+  mrg_queue_draw (e->mrg, NULL);
+}
+
+
+static gchar *edited_prop = NULL;
+
+static void update_prop (const char *new_string, GeglNode *node)
+{
+  gegl_node_set (node, edited_prop, new_string, NULL);
+}
+
+static void set_edited_prop (MrgEvent *e, void *data1, void *data2)
+{
+  if (edited_prop)
+    g_free (edited_prop);
+  edited_prop = g_strdup (data2);
+  mrg_event_stop_propagate (e);
+
+  mrg_queue_draw (e->mrg, NULL);
+}
+
+static void unset_edited_prop (MrgEvent *e, void *data1, void *data2)
+{
+  if (edited_prop)
+    g_free (edited_prop);
+  edited_prop = NULL;
+  mrg_event_stop_propagate (e);
 
   mrg_queue_draw (e->mrg, NULL);
 }
@@ -489,7 +491,7 @@ static void draw_gegl_generic (State *state, Mrg *mrg, cairo_t *cr, GeglNode *no
   mrg_set_edge_left (mrg, mrg_width (mrg) * 0.1);
   mrg_set_edge_top  (mrg, mrg_height (mrg) * 0.1);
   mrg_set_font_size (mrg, mrg_height (mrg) * 0.07);
-  mrg_set_style (mrg, "color:white; background-color: transparent");
+  mrg_set_style (mrg, "color:white; background-color: rgba(0,0,0,0.5)");
 
   cairo_save (cr);
   mrg_printf (mrg, "%s\n", op_name);
@@ -506,14 +508,16 @@ static void draw_gegl_generic (State *state, Mrg *mrg, cairo_t *cr, GeglNode *no
       {
         if (g_type_is_a (pspecs[i]->value_type, G_TYPE_DOUBLE) ||
             g_type_is_a (pspecs[i]->value_type, G_TYPE_INT) ||
-            g_type_is_a (pspecs[i]->value_type, G_TYPE_STRING) ||
             g_type_is_a (pspecs[i]->value_type, G_TYPE_BOOLEAN))
           tot_pos ++;
+        else if (g_type_is_a (pspecs[i]->value_type, G_TYPE_STRING))
+          tot_pos +=3;
       }
 
       for (gint i = 0; i < n_props; i++)
       {
-        mrg_set_xy (mrg, mrg_em(mrg), mrg_height (mrg) - mrg_em (mrg) * ((tot_pos-pos_no)));
+        mrg_set_xy (mrg, mrg_em(mrg),
+                    mrg_height (mrg) - mrg_em (mrg) * ((tot_pos-pos_no)));
 
         if (g_type_is_a (pspecs[i]->value_type, G_TYPE_DOUBLE))
         {
@@ -542,8 +546,26 @@ static void draw_gegl_generic (State *state, Mrg *mrg, cairo_t *cr, GeglNode *no
         }
         else if (g_type_is_a (pspecs[i]->value_type, G_TYPE_INT))
         {
+          float xpos;
+          GeglParamSpecInt *geglspec = (void*)pspecs[i];
           gint value;
           gegl_node_get (node, pspecs[i]->name, &value, NULL);
+
+          cairo_rectangle (cr, 0,
+             mrg_height (mrg) - mrg_em (mrg) * ((tot_pos-pos_no+1)),
+             mrg_width (mrg), mrg_em(mrg));
+          cairo_set_source_rgba (cr, 0,0,0, 0.5);
+
+          mrg_listen (mrg, MRG_DRAG, prop_int_drag_cb, node,(void*)pspecs[i]);
+
+          cairo_fill (cr);
+          xpos = (value - geglspec->ui_minimum) / (1.0 + geglspec->ui_maximum - geglspec->ui_minimum);
+          cairo_rectangle (cr, xpos * mrg_width(mrg) - mrg_em(mrg)/4,
+              mrg_height (mrg) - mrg_em (mrg) * ((tot_pos-pos_no+1)),
+              mrg_em(mrg)/2, mrg_em(mrg));
+          cairo_set_source_rgba (cr, 1,1,1, 0.5);
+          cairo_fill (cr);
+
           mrg_printf (mrg, "%s:%i\n", pspecs[i]->name, value);
           pos_no ++;
         }
@@ -551,9 +573,26 @@ static void draw_gegl_generic (State *state, Mrg *mrg, cairo_t *cr, GeglNode *no
         {
           char *value = NULL;
           gegl_node_get (node, pspecs[i]->name, &value, NULL);
-          pos_no ++;
-          mrg_printf (mrg, "%s:%s\n", pspecs[i]->name, value);
-          if (value) g_free (value);
+          pos_no +=3;
+
+          if (edited_prop && !strcmp (edited_prop, pspecs[i]->name))
+          {
+            mrg_printf (mrg, "%s:", pspecs[i]->name);
+            mrg_text_listen (mrg, MRG_CLICK, unset_edited_prop, node, pspecs[i]->name);
+            mrg_edit_start (mrg, update_prop, node);
+            mrg_printf (mrg, "%s\n", value);
+            mrg_edit_end (mrg);
+            mrg_text_listen_done (mrg);
+          }
+          else
+          {
+            mrg_text_listen (mrg, MRG_CLICK, set_edited_prop, node, pspecs[i]->name);
+            mrg_printf (mrg, "%s:%s\n", pspecs[i]->name, value);
+            mrg_text_listen_done (mrg);
+          }
+
+          if (value)
+            g_free (value);
         }
         else if (g_type_is_a (pspecs[i]->value_type, G_TYPE_BOOLEAN))
         {
@@ -901,10 +940,11 @@ static void ui_viewer (State *o)
   Mrg *mrg = o->mrg;
   cairo_t *cr = mrg_cr (mrg);
   cairo_rectangle (cr, 0,0, mrg_width(mrg), mrg_height(mrg));
+#if 0
   mrg_listen (mrg, MRG_DRAG, on_pan_drag, o, NULL);
   mrg_listen (mrg, MRG_MOTION, on_viewer_motion, o, NULL);
-
   mrg_listen (mrg, MRG_SCROLL, scroll_cb, o, NULL);
+#endif
 
   cairo_scale (cr, mrg_width(mrg), mrg_height(mrg));
   cairo_new_path (cr);
@@ -960,18 +1000,22 @@ static void ui_viewer (State *o)
   mrg_add_binding (mrg, "right", NULL, NULL, pan_right_cb, o);
   mrg_add_binding (mrg, "up", NULL, NULL,    pan_up_cb, o);
   mrg_add_binding (mrg, "down", NULL, NULL,  pan_down_cb, o);
-  mrg_add_binding (mrg, "+", NULL, NULL,     zoom_in_cb, o);
-  mrg_add_binding (mrg, "=", NULL, NULL,     zoom_in_cb, o);
-  mrg_add_binding (mrg, "-", NULL, NULL,     zoom_out_cb, o);
-  mrg_add_binding (mrg, "1", NULL, NULL,     zoom_1_cb, o);
-  mrg_add_binding (mrg, "m", NULL, NULL,     zoom_fit_cb, o);
-  mrg_add_binding (mrg, "m", NULL, NULL,     zoom_fit_cb, o);
-  mrg_add_binding (mrg, "x", NULL, NULL,     discard_cb, o);
 
-  mrg_add_binding (mrg, "space", NULL, NULL,     go_next_cb , o);
-  mrg_add_binding (mrg, "n", NULL, NULL,         go_next_cb, o);
-  mrg_add_binding (mrg, "p", NULL, NULL,         go_prev_cb, o);
-  mrg_add_binding (mrg, "backspace", NULL, NULL, go_prev_cb, o);
+  if (!edited_prop)
+  {
+    mrg_add_binding (mrg, "+", NULL, NULL,     zoom_in_cb, o);
+    mrg_add_binding (mrg, "=", NULL, NULL,     zoom_in_cb, o);
+    mrg_add_binding (mrg, "-", NULL, NULL,     zoom_out_cb, o);
+    mrg_add_binding (mrg, "1", NULL, NULL,     zoom_1_cb, o);
+    mrg_add_binding (mrg, "m", NULL, NULL,     zoom_fit_cb, o);
+    mrg_add_binding (mrg, "m", NULL, NULL,     zoom_fit_cb, o);
+    mrg_add_binding (mrg, "x", NULL, NULL,     discard_cb, o);
+
+    mrg_add_binding (mrg, "space", NULL, NULL,     go_next_cb , o);
+    mrg_add_binding (mrg, "n", NULL, NULL,         go_next_cb, o);
+    mrg_add_binding (mrg, "p", NULL, NULL,         go_prev_cb, o);
+    mrg_add_binding (mrg, "backspace", NULL, NULL, go_prev_cb, o);
+  }
 
   if (o->slide_enabled && o->slide_timeout == 0)
   {
@@ -1004,11 +1048,6 @@ static void gegl_ui (Mrg *mrg, void *data)
   if (o->is_video)
    {
      o->frame_no++;
-     if (g_getenv ("GEGL_UI_DEBUG_SEEK"))
-     {
-       if ((o->frame_no / 200) % 2 == 1)
-         o->frame_no+=600;
-     }
      fprintf (stderr, "\r%i", o->frame_no);
      gegl_node_set (o->load, "frame", o->frame_no, NULL);
      mrg_queue_draw (o->mrg, NULL);
@@ -1016,11 +1055,11 @@ static void gegl_ui (Mrg *mrg, void *data)
 
   mrg_gegl_blit (mrg,
 		 0, 0,
-                 mrg_width (mrg), mrg_height (mrg),
+         mrg_width (mrg), mrg_height (mrg),
 		 o->sink,
 		 o->u, o->v,
 		 o->scale,
-                 o->render_quality);
+         o->render_quality);
 
   if (o->is_video)
   {
@@ -1060,6 +1099,12 @@ static void gegl_ui (Mrg *mrg, void *data)
 #endif
   }
 
+  cairo_rectangle (mrg_cr (mrg), 0,0, mrg_width(mrg), mrg_height(mrg));
+  mrg_listen (mrg, MRG_DRAG, on_pan_drag, o, NULL);
+  mrg_listen (mrg, MRG_MOTION, on_viewer_motion, o, NULL);
+  mrg_listen (mrg, MRG_SCROLL, scroll_cb, o, NULL);
+  cairo_new_path (mrg_cr (mrg));
+
   if (o->show_actions)
   {
     ui_actions (o);
@@ -1074,6 +1119,11 @@ static void gegl_ui (Mrg *mrg, void *data)
   else
   {
     struct stat stat_buf;
+
+  if (edited_prop)
+    g_free (edited_prop);
+  edited_prop = NULL;
+
     lstat (o->path, &stat_buf);
     if (S_ISREG (stat_buf.st_mode))
     {
@@ -1090,13 +1140,17 @@ static void gegl_ui (Mrg *mrg, void *data)
   }
 
   mrg_add_binding (mrg, "control-q", NULL, NULL, mrg_quit_cb, o);
-  mrg_add_binding (mrg, "q", NULL, NULL,         mrg_quit_cb, o);
-  mrg_add_binding (mrg, "f", NULL, NULL,         toggle_fullscreen_cb, o);
   mrg_add_binding (mrg, "F11", NULL, NULL,       toggle_fullscreen_cb, o);
-  mrg_add_binding (mrg, "tab", NULL, NULL,       toggle_show_controls_cb, o);
-  mrg_add_binding (mrg, "s", NULL, NULL,         toggle_slideshow_cb, o);
-  mrg_add_binding (mrg, ",", NULL, NULL,         preview_less_cb, o);
-  mrg_add_binding (mrg, ".", NULL, NULL,         preview_more_cb, o);
+
+  if (!edited_prop)
+  {
+    mrg_add_binding (mrg, "tab", NULL, NULL,       toggle_show_controls_cb, o);
+    mrg_add_binding (mrg, "q", NULL, NULL,         mrg_quit_cb, o);
+    mrg_add_binding (mrg, "f", NULL, NULL,         toggle_fullscreen_cb, o);
+    mrg_add_binding (mrg, "s", NULL, NULL,         toggle_slideshow_cb, o);
+    mrg_add_binding (mrg, ",", NULL, NULL,         preview_less_cb, o);
+    mrg_add_binding (mrg, ".", NULL, NULL,         preview_more_cb, o);
+  }
 }
 
 /***********************************************/
@@ -1289,7 +1343,7 @@ static void load_path (State *o)
   o->frame_no = 0;
   o->prev_frame_played = 0;
 
-  if (str_has_video_suffix (path))
+  if (gegl_str_has_video_suffix (path))
   {
     o->is_video = 1;
     o->gegl = gegl_node_new ();
@@ -1309,7 +1363,7 @@ static void load_path (State *o)
     {
       GSList *nodes, *n;
       char *containing_path = get_path_parent (o->path);
-      o->gegl = gegl_node_new_from_xml (meta, containing_path);
+      o->gegl = gegl_node_new_from_serialized (meta, containing_path);
       free (containing_path);
       o->sink = gegl_node_new_child (o->gegl,
                        "operation", "gegl:nop", NULL);
@@ -1369,12 +1423,17 @@ static void load_path (State *o)
         zoom_to_fit (o);
     }
   }
-
   if (o->ops)
   {
-    gegl_create_chain(o->ops,
-                      gegl_node_get_producer (o->sink, "input", NULL),
-                      o->sink);
+    GError *error = NULL;
+    gegl_create_chain_argv (o->ops,
+                    gegl_node_get_producer (o->sink, "input", NULL),
+                    o->sink, 0, gegl_node_get_bounding_box (o->sink).height,
+                    &error);
+    if (error)
+    {
+      fprintf (stderr, "Error: %s\n", error->message);
+    }
   }
 
   mrg_queue_draw (o->mrg, NULL);
@@ -1896,7 +1955,7 @@ static void save_cb (MrgEvent *event, void *data1, void *data2)
   GeglNode *load;
   State *o = data1;
   gchar *path;
-  char *xml;
+  char *serialized;
 
   gegl_node_link_many (o->sink, o->save, NULL);
   gegl_node_process (o->save);
@@ -1909,13 +1968,13 @@ static void save_cb (MrgEvent *event, void *data1, void *data2)
   gegl_node_link_many (load, o->source, NULL);
   {
     char *containing_path = get_path_parent (o->path);
-    xml = gegl_node_to_xml (o->sink, containing_path);
+    serialized = gegl_serialize (NULL, o->sink, containing_path);
     free (containing_path);
   }
   gegl_node_remove_child (o->gegl, load);
   gegl_node_link_many (o->load, o->source, NULL);
-  gegl_meta_set (path, xml);
-  g_free (xml);
+  gegl_meta_set (path, serialized);
+  g_free (serialized);
   o->rev = 0;
 }
 
