@@ -24,15 +24,6 @@ static const char* op_c_source =
 "                                                                              \n"
 "#ifdef GEGL_PROPERTIES                                                        \n"
 "                                                                              \n"
-"enum_start (gegl_dither_strategy)                                             \n"
-"  enum_value (GEGL_DITHER_NONE,             \"none\",             N_(\"None\"))\n"
-"  enum_value (GEGL_DITHER_RANDOM,           \"random\",           N_(\"Random\"))\n"
-"  enum_value (GEGL_DITHER_RESILIENT,        \"resilient\",        N_(\"Resilient\"))\n"
-"  enum_value (GEGL_DITHER_RANDOM_COVARIANT, \"random-covariant\", N_(\"Random Covariant\"))\n"
-"  enum_value (GEGL_DITHER_BAYER,            \"bayer\",            N_(\"Bayer\"))\n"
-"  enum_value (GEGL_DITHER_FLOYD_STEINBERG,  \"floyd-steinberg\",  N_(\"Floyd-Steinberg\"))\n"
-"enum_end (GeglDitherStrategy)                                                 \n"
-"                                                                              \n"
 "property_int  (red_bits, _(\"Red bits\"), 8)                                  \n"
 "    description(_(\"Number of bits for red channel\"))                        \n"
 "    value_range (1, 16)                                                       \n"
@@ -49,15 +40,16 @@ static const char* op_c_source =
 "    description(_(\"Number of bits for alpha channel\"))                      \n"
 "    value_range (1, 16)                                                       \n"
 "                                                                              \n"
-"property_enum (dither_strategy, _(\"Dithering strategy\"),                    \n"
-"               GeglDitherStrategy, gegl_dither_strategy, GEGL_DITHER_RESILIENT)\n"
-"    description (_(\"The dithering strategy to use\"))                        \n"
+"property_enum (dither_method, _(\"Dithering method\"),                        \n"
+"               GeglDitherMethod, gegl_dither_method, GEGL_DITHER_RESILIENT)   \n"
+"    description (_(\"The dithering method to use\"))                          \n"
 "                                                                              \n"
 "property_seed (seed, _(\"Random seed\"), rand)                                \n"
 "                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
 "#define GEGL_OP_FILTER                                                        \n"
+"#define GEGL_OP_NAME     color_reduction                                      \n"
 "#define GEGL_OP_C_SOURCE color-reduction.c                                    \n"
 "                                                                              \n"
 "#include \"gegl-op.h\"                                                        \n"
@@ -81,7 +73,7 @@ static const char* op_c_source =
 "    channel_mask [i] = ~((1 << (16 - channel_bits [i])) - 1);                 \n"
 "}                                                                             \n"
 "                                                                              \n"
-"static guint                                                                  \n"
+"static inline guint                                                           \n"
 "quantize_value (guint value,                                                  \n"
 "                guint n_bits,                                                 \n"
 "                guint mask)                                                   \n"
@@ -218,7 +210,7 @@ static const char* op_c_source =
 "  43, 27, 39, 23, 42, 26, 38, 22                                              \n"
 "};                                                                            \n"
 "                                                                              \n"
-"static void                                                                   \n"
+"static void inline                                                            \n"
 "process_row_bayer (GeglBufferIterator *gi,                                    \n"
 "                   guint               channel_mask [4],                      \n"
 "                   guint               channel_bits [4],                      \n"
@@ -252,7 +244,146 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
-"static void                                                                   \n"
+"static void inline                                                            \n"
+"process_row_arithmetic_add (GeglBufferIterator *gi,                           \n"
+"                            guint               channel_mask [4],             \n"
+"                            guint               channel_bits [4],             \n"
+"                            gint                y)                            \n"
+"{                                                                             \n"
+"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
+"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
+"  guint x;                                                                    \n"
+"  for (x = 0; x < gi->roi->width; x++)                                        \n"
+"    {                                                                         \n"
+"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
+"      guint ch;                                                               \n"
+"                                                                              \n"
+"      for (ch = 0; ch < 4; ch++)                                              \n"
+"        {                                                                     \n"
+"          gint u = gi->roi->x + x;                                            \n"
+"          gint v = gi->roi->y + y;                                            \n"
+"          gfloat mask;                                                        \n"
+"          gfloat value;                                                       \n"
+"          gfloat value_clamped;                                               \n"
+"          gfloat quantized;                                                   \n"
+"          mask         = (((u+ch * 67) + v * 236) * 119) & 255;               \n"
+"          mask         = ((mask - 128) * 65536.0 / 256.0) / (1 << (channel_bits [ch] - 1));\n"
+"          value         = data_in [pixel + ch] + mask;                        \n"
+"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
+"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
+"                                          channel_bits [ch],                  \n"
+"                                          channel_mask [ch]);                 \n"
+"                                                                              \n"
+"          data_out [pixel + ch] = (guint16) quantized;                        \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void inline                                                            \n"
+"process_row_arithmetic_xor (GeglBufferIterator *gi,                           \n"
+"                            guint               channel_mask [4],             \n"
+"                            guint               channel_bits [4],             \n"
+"                            gint                y)                            \n"
+"{                                                                             \n"
+"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
+"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
+"  guint x;                                                                    \n"
+"  for (x = 0; x < gi->roi->width; x++)                                        \n"
+"    {                                                                         \n"
+"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
+"      guint ch;                                                               \n"
+"                                                                              \n"
+"      for (ch = 0; ch < 4; ch++)                                              \n"
+"        {                                                                     \n"
+"          gint u = gi->roi->x + x;                                            \n"
+"          gint v = gi->roi->y + y;                                            \n"
+"          gfloat mask;                                                        \n"
+"          gfloat value;                                                       \n"
+"          gfloat value_clamped;                                               \n"
+"          gfloat quantized;                                                   \n"
+"          mask         = (((u+ch * 17) ^ v * 149) * 1234) & 511;              \n"
+"          mask         = ((mask - 257) * 65536.0 / 512.0) / (1 << (channel_bits [ch] - 1));\n"
+"          value         = data_in [pixel + ch] + mask;                        \n"
+"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
+"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
+"                                          channel_bits [ch],                  \n"
+"                                          channel_mask [ch]);                 \n"
+"                                                                              \n"
+"          data_out [pixel + ch] = (guint16) quantized;                        \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void inline                                                            \n"
+"process_row_arithmetic_add_covariant (GeglBufferIterator *gi,                 \n"
+"                                      guint               channel_mask [4],   \n"
+"                                      guint               channel_bits [4],   \n"
+"                                      gint                y)                  \n"
+"{                                                                             \n"
+"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
+"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
+"  guint x;                                                                    \n"
+"  for (x = 0; x < gi->roi->width; x++)                                        \n"
+"    {                                                                         \n"
+"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
+"      guint ch;                                                               \n"
+"                                                                              \n"
+"      for (ch = 0; ch < 4; ch++)                                              \n"
+"        {                                                                     \n"
+"          gint u = gi->roi->x + x;                                            \n"
+"          gint v = gi->roi->y + y;                                            \n"
+"          gfloat mask;                                                        \n"
+"          gfloat value;                                                       \n"
+"          gfloat value_clamped;                                               \n"
+"          gfloat quantized;                                                   \n"
+"          mask         = ((u+ v * 236) * 119) & 255;                          \n"
+"          mask         = ((mask - 128) * 65536.0 / 256.0) / (1 << (channel_bits [ch] - 1));\n"
+"          value         = data_in [pixel + ch] + mask;                        \n"
+"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
+"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
+"                                          channel_bits [ch],                  \n"
+"                                          channel_mask [ch]);                 \n"
+"                                                                              \n"
+"          data_out [pixel + ch] = (guint16) quantized;                        \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void inline                                                            \n"
+"process_row_arithmetic_xor_covariant (GeglBufferIterator *gi,                 \n"
+"                                      guint               channel_mask [4],   \n"
+"                                      guint               channel_bits [4],   \n"
+"                                      gint                y)                  \n"
+"{                                                                             \n"
+"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
+"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
+"  guint x;                                                                    \n"
+"  for (x = 0; x < gi->roi->width; x++)                                        \n"
+"    {                                                                         \n"
+"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
+"      guint ch;                                                               \n"
+"                                                                              \n"
+"      for (ch = 0; ch < 4; ch++)                                              \n"
+"        {                                                                     \n"
+"          gint u = gi->roi->x + x;                                            \n"
+"          gint v = gi->roi->y + y;                                            \n"
+"          gfloat mask;                                                        \n"
+"          gfloat value;                                                       \n"
+"          gfloat value_clamped;                                               \n"
+"          gfloat quantized;                                                   \n"
+"          mask         = ((u ^ v * 149) * 1234) & 511;                        \n"
+"          mask         = ((mask - 257) * 65536.0 / 512.0) / (1 << (channel_bits [ch] - 1));\n"
+"          value         = data_in [pixel + ch] + mask;                        \n"
+"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
+"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
+"                                          channel_bits [ch],                  \n"
+"                                          channel_mask [ch]);                 \n"
+"                                                                              \n"
+"          data_out [pixel + ch] = (guint16) quantized;                        \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"}                                                                             \n"
+"static void inline                                                            \n"
 "process_row_random_covariant (GeglBufferIterator *gi,                         \n"
 "                              guint               channel_mask [4],           \n"
 "                              guint               channel_bits [4],           \n"
@@ -270,9 +401,9 @@ static const char* op_c_source =
 "                                             gi->roi->y + y, 0, 0));          \n"
 "      for (ch = 0; ch < 4; ch++)                                              \n"
 "        {                                                                     \n"
-"          gdouble value;                                                      \n"
-"          gdouble value_clamped;                                              \n"
-"          gdouble quantized;                                                  \n"
+"          gfloat value;                                                       \n"
+"          gfloat value_clamped;                                               \n"
+"          gfloat quantized;                                                   \n"
 "                                                                              \n"
 "          value         = data_in [pixel + ch] + (r / (1 << channel_bits [ch]));\n"
 "          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
@@ -285,7 +416,7 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
-"static void                                                                   \n"
+"static void inline                                                            \n"
 "process_row_random (GeglBufferIterator *gi,                                   \n"
 "                    guint               channel_mask [4],                     \n"
 "                    guint               channel_bits [4],                     \n"
@@ -318,7 +449,7 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
-"static void                                                                   \n"
+"static void inline                                                            \n"
 "process_row_resilient (GeglBufferIterator *gi,                                \n"
 "                       guint               channel_mask [4],                  \n"
 "                       guint               channel_bits [4],                  \n"
@@ -352,7 +483,7 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
-"static void                                                                   \n"
+"static void inline                                                            \n"
 "process_row_no_dither (GeglBufferIterator *gi,                                \n"
 "                       guint               channel_mask [4],                  \n"
 "                       guint               channel_bits [4],                  \n"
@@ -380,7 +511,7 @@ static const char* op_c_source =
 "                  const GeglRectangle *result,                                \n"
 "                  guint               *channel_bits,                          \n"
 "                  GeglRandom          *rand,                                  \n"
-"                  GeglDitherStrategy   dither_strategy)                       \n"
+"                  GeglDitherMethod     dither_method)                         \n"
 "{                                                                             \n"
 "  GeglBufferIterator *gi;                                                     \n"
 "  guint               channel_mask [4];                                       \n"
@@ -396,31 +527,50 @@ static const char* op_c_source =
 "  while (gegl_buffer_iterator_next (gi))                                      \n"
 "    {                                                                         \n"
 "      guint    y;                                                             \n"
-"      for (y = 0; y < gi->roi->height; y++)                                   \n"
+"      switch (dither_method)                                                  \n"
 "        {                                                                     \n"
-"          switch (dither_strategy)                                            \n"
-"            {                                                                 \n"
-"            case GEGL_DITHER_NONE:                                            \n"
+"          case GEGL_DITHER_NONE:                                              \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_no_dither (gi, channel_mask, channel_bits, y);      \n"
-"              break;                                                          \n"
-"            case GEGL_DITHER_RANDOM:                                          \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_RANDOM:                                            \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_random (gi, channel_mask, channel_bits, y, rand);   \n"
-"              break;                                                          \n"
-"            case GEGL_DITHER_RESILIENT:                                       \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_RESILIENT:                                         \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_resilient (gi, channel_mask, channel_bits, y, rand);\n"
-"              break;                                                          \n"
-"            case GEGL_DITHER_RANDOM_COVARIANT:                                \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_RANDOM_COVARIANT:                                  \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_random_covariant (gi, channel_mask, channel_bits, y, rand);\n"
-"              break;                                                          \n"
-"            case GEGL_DITHER_BAYER:                                           \n"
+"             break;                                                           \n"
+"          case GEGL_DITHER_BAYER:                                             \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_bayer (gi, channel_mask, channel_bits, y);          \n"
-"              break;                                                          \n"
-"            case GEGL_DITHER_FLOYD_STEINBERG:                                 \n"
-"              /* Done separately */                                           \n"
-"              break;                                                          \n"
-"            default:                                                          \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_FLOYD_STEINBERG:                                   \n"
+"            /* Done separately */                                             \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_ARITHMETIC_ADD:                                    \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
+"              process_row_arithmetic_add (gi, channel_mask, channel_bits, y); \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_ARITHMETIC_XOR:                                    \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
+"              process_row_arithmetic_xor (gi, channel_mask, channel_bits, y); \n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_ARITHMETIC_ADD_COVARIANT:                          \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
+"              process_row_arithmetic_add_covariant (gi, channel_mask, channel_bits, y);\n"
+"            break;                                                            \n"
+"          case GEGL_DITHER_ARITHMETIC_XOR_COVARIANT:                          \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
+"              process_row_arithmetic_xor_covariant (gi, channel_mask, channel_bits, y);\n"
+"            break;                                                            \n"
+"          default:                                                            \n"
+"            for (y = 0; y < gi->roi->height; y++)                             \n"
 "              process_row_no_dither (gi, channel_mask, channel_bits, y);      \n"
-"            }                                                                 \n"
 "        }                                                                     \n"
 "    }                                                                         \n"
 "}                                                                             \n"
@@ -432,7 +582,7 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  GeglProperties *o = GEGL_PROPERTIES (self);                                 \n"
 "                                                                              \n"
-"  if (o->dither_strategy == GEGL_DITHER_FLOYD_STEINBERG)                      \n"
+"  if (o->dither_method == GEGL_DITHER_FLOYD_STEINBERG)                        \n"
 "    return *gegl_operation_source_get_bounding_box (self, \"input\");         \n"
 "  else                                                                        \n"
 "    return *roi;                                                              \n"
@@ -444,7 +594,7 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  GeglProperties *o = GEGL_PROPERTIES (self);                                 \n"
 "                                                                              \n"
-"  if (o->dither_strategy == GEGL_DITHER_FLOYD_STEINBERG)                      \n"
+"  if (o->dither_method == GEGL_DITHER_FLOYD_STEINBERG)                        \n"
 "    return *gegl_operation_source_get_bounding_box (self, \"input\");         \n"
 "  else                                                                        \n"
 "    return *roi;                                                              \n"
@@ -465,9 +615,9 @@ static const char* op_c_source =
 "  channel_bits [2] = o->blue_bits;                                            \n"
 "  channel_bits [3] = o->alpha_bits;                                           \n"
 "                                                                              \n"
-"  if (o->dither_strategy != GEGL_DITHER_FLOYD_STEINBERG)                      \n"
+"  if (o->dither_method != GEGL_DITHER_FLOYD_STEINBERG)                        \n"
 "    process_standard (input, output, result, channel_bits,                    \n"
-"                      o->rand, o->dither_strategy);                           \n"
+"                      o->rand, o->dither_method);                             \n"
 "  else                                                                        \n"
 "    process_floyd_steinberg (input, output, result, channel_bits);            \n"
 "                                                                              \n"
@@ -487,7 +637,7 @@ static const char* op_c_source =
 "    \"    <param name='green-bits'>2</param>\"                                \n"
 "    \"    <param name='blue-bits'>2</param>\"                                 \n"
 "    \"    <param name='alpha-bits'>2</param>\"                                \n"
-"    \"    <param name='dither-strategy'>floyd-steinberg</param>\"             \n"
+"    \"    <param name='dither-method'>floyd-steinberg</param>\"               \n"
 "    \"  </params>\"                                                           \n"
 "    \"</node>\"                                                               \n"
 "    \"<node operation='gegl:load'>\"                                          \n"
