@@ -67,6 +67,7 @@ static const char* op_c_source =
 "#else                                                                         \n"
 "                                                                              \n"
 "#define GEGL_OP_FILTER                                                        \n"
+"#define GEGL_OP_NAME     gblur_1d                                             \n"
 "#define GEGL_OP_C_SOURCE gblur-1d.c                                           \n"
 "                                                                              \n"
 "#include \"gegl-op.h\"                                                        \n"
@@ -821,7 +822,10 @@ static const char* op_c_source =
 "          format == babl_format (\"RaGaBaA float\"))                          \n"
 "        if (fir_cl_process(input, output, result, format,                     \n"
 "                           cmatrix, clen, o->orientation, abyss_policy))      \n"
+"        {                                                                     \n"
+"          gegl_free (cmatrix);                                                \n"
 "          return TRUE;                                                        \n"
+"        }                                                                     \n"
 "                                                                              \n"
 "      if (o->orientation == GEGL_ORIENTATION_HORIZONTAL)                      \n"
 "        fir_hor_blur (input, result, output, cmatrix, clen, abyss_policy, format);\n"
